package BPpath.poddroid;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.app.ProgressDialog;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.MediaStore.MediaColumns;
import android.view.Gravity;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import com.gowtham.music.metadata.IMusicMetadata;
import com.gowtham.music.metadata.MusicMetadata;
import com.gowtham.music.metadata.MusicMetadataSet;
import com.gowtham.music.myid3.MyID3;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

/*
 * MP3Tagger.java is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (gnudroider@tuxfamily.org) (c) 2016
 * Original file Copyright (c) 2013 gowtham
 *
 */

public class MP3Tagger extends Activity {

	private static final int REQUEST_CODE = 1;
	private Button buttonLoadAudioFile, buttonSaveTags;
	private TextView textViewAudioFilePath, editTextTitle, editTextComment, editTextArtist, editTextYear;
	
	private String audioFilePath;
	private MusicMetadataSet metadataSet;
	private IMusicMetadata metadata;
	private ProgressDialog progressDialog;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_mp3tagger);
		buttonLoadAudioFile = (Button) findViewById(R.id.buttonLoadAudioFile);
		buttonSaveTags = (Button) findViewById(R.id.buttonSave);
		textViewAudioFilePath = (TextView) findViewById(R.id.textViewAudioFilePath);
		editTextTitle = (TextView) findViewById(R.id.editTextTitle);
		editTextComment = (TextView) findViewById(R.id.editTextComment);
		editTextArtist = (TextView) findViewById(R.id.editTextArtist);
		editTextYear = (TextView) findViewById(R.id.editTextYear);
		Intent intent = getIntent();
		audioFilePath = intent.getStringExtra("FNAME");
		loadTags(audioFilePath);
	}
	

	public void showFileBrowser(View view) {
		Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
		intent.setType("audio/mpeg");
		startActivityForResult(intent, REQUEST_CODE);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent intent) {
		if(resultCode == RESULT_OK) {
			Uri uri = (Uri)intent.getData();
			String path = uri.getPath();
			if (path.contains("/media/")){
				path = uri2fname(uri);
			}
			loadTags(path);
		}
	}

	private String uri2fname(Uri uri) {
		String[] proj = { MediaColumns.DATA };
		Cursor cursor = managedQuery(uri, proj, null, null, null);
		int column_index = cursor.getColumnIndexOrThrow(MediaColumns.DATA);
		cursor.moveToFirst();
		return cursor.getString(column_index);
	}

	private void loadTagsInternal(String path) {
		textViewAudioFilePath.setText(path);
		TagsReaderAsyncTask task = new TagsReaderAsyncTask();
		progressDialog = new ProgressDialog(this, ProgressDialog.STYLE_SPINNER);
		progressDialog.setTitle("Loading tags...");
		progressDialog.show();
		task.execute(new File(path));
	}
	
	private void showTags(IMusicMetadata metadata) {
		clearTags();
		editTextTitle.setText(metadata.getSongTitle());
		editTextArtist.setText(metadata.getArtist());
		String license = metadata.getComment();
		if ((license == null) || (license.equals("")))
			license = "Creative Commons 3.0 SA-BY-NC";
		editTextComment.setText(license);
		editTextYear.setText(metadata.getYear());
	}
	
	private void clearTags() {
		editTextTitle.setText("");
		editTextComment.setText("");
		editTextArtist.setText("");
		editTextYear.setText("");
	}

	private void loadTags(String path) {
			audioFilePath = path;
			loadTagsInternal(path);
	}
	
	private void error(Exception e) {
		AlertDialog.Builder b = new Builder(this);
		b.setTitle("ERROR: " + e.getMessage());
		b.setCancelable(true);
		b.create().show();
		e.printStackTrace();
	}
	
	public void saveTags(View view) {
		if(audioFilePath == null || metadata == null || metadataSet == null)
			return;
		try {
			saveTagsInternal();
		}
		catch(Exception e) {
			error(e);
		}
	}

	private void saveTagsInternal() throws Exception {
		progressDialog = new ProgressDialog(this, ProgressDialog.STYLE_SPINNER);
		progressDialog.setTitle("Saving tags...");
		progressDialog.show();
		TagsWriterAsyncTask task = new TagsWriterAsyncTask();
		task.execute((Void[])null);
	}	
	
	private void copy(File src, File dst) throws IOException {
		InputStream in = null;
	    OutputStream out = null;
	    int len;
	    byte []buf;
	    
		try {
		    in = new FileInputStream(src);
		    out = new FileOutputStream(dst);
	
		    buf = new byte[1024];
		    while ((len = in.read(buf)) > 0)
		        out.write(buf, 0, len);
		}
		finally {
			if(in != null)
				in.close();
			if(out != null)
				out.close();
		}
	}
	
	// ??? PRAFileHandler's copy (move) does not work here. Find out why. 
	private void move(File src, File dst) throws IOException {
		copy(src, dst);
		src.delete();
	}

	
	/** Manages reading tags asynchronously in the background 
	 * 
	 * @author Gowtham
	 */
	private class TagsReaderAsyncTask extends AsyncTask<File, String, MusicMetadataSet> {

		private Exception e;
		
		@Override
		protected MusicMetadataSet doInBackground(File... files) {
			try {
				publishProgress("Reading audio file...");
				MyID3 id3 = new MyID3();
				metadataSet = id3.read(files[0]);
				publishProgress("Done");
			}
			catch(IOException e) {
				this.e = e;
			}
			return metadataSet;
		}
		
		@Override
		protected void onProgressUpdate(String... msgs) {
			progressDialog.setMessage(msgs[0]);
		}
		
		@Override
		protected void onPostExecute(MusicMetadataSet result) {
			progressDialog.dismiss();
			if(result == null) {
				error(getException());
			} else {
				metadataSet = result;
				metadata = metadataSet.getSimplified();
				showTags(metadata);
			}
		}
		
		public Exception getException() {
			return e;
		}
	}
	
	/** Manages writing tags asynchronously in the background
	 * 
	 * @author Gowtham
	 *
	 */
	private class TagsWriterAsyncTask extends AsyncTask<Void,String,Object> {

		private Exception e;
		
		@Override
		protected void onPreExecute() {
			metadata.setSongTitle(editTextTitle.getText().toString().trim());
			metadata.setComment(editTextComment.getText().toString().trim());
			metadata.setArtist(editTextArtist.getText().toString().trim());
			metadata.setYear(editTextYear.getText().toString().trim());
		}
		
		@Override
		protected Object doInBackground(Void... arg0) {
			
			try {
				publishProgress("Preparing files...");
				File inputFile = new File(audioFilePath);
				File cacheDir = getCacheDir();
				File tempFile = File.createTempFile(inputFile.getName(), "temp", cacheDir);
				
				publishProgress("Compiling and writing tags...");
				new MyID3().write(inputFile, tempFile, metadataSet, (MusicMetadata) metadata);
				
				publishProgress("Moving file");
				move(tempFile, inputFile);
				
				publishProgress("Done");
			} catch(Exception e) {
				this.e = e;
			}
			
			return null;
		}
		
		@Override
		protected void onProgressUpdate(String... msg) {
			progressDialog.setMessage(msg[0]);
		}
		
		@Override
		protected void onPostExecute(Object result) {
			progressDialog.dismiss();
			if(e != null)
				error(e);
		}
		
		public Exception getException() {
			return this.e;
		}
	}

	public void toaster(String text, int length) {
		Toast t = Toast.makeText(this, text, length);
		t.setGravity(Gravity.CENTER, 0, 0);
		t.show();
	}


}
